<?php

if (! class_exists('EG_Series')) {

	/**
	 * Class EG_Series
	 *
	 * Implement a shortcode to display the posts series
	 *
	 */
	Class EG_Series extends EG_Plugin_102 {

		var $series_data;

		// Default options for shortcode [series] (list of all series)
		var $eg_series_shortcode_series_defaults = array(
			'title'	   => '',
			'titletag' => 'h2',
			'listtype' => 'ul',
			'number'   => 0,
			'more'     => 0
		);

		// Default options for shortcode [seriesposts] (list of all posts of a specified serie)
		var $eg_series_shortcode_posts_defaults = array(
			'id'	   => 0,
			'title'	   => '',
			'titletag' => 'h2',
			'listtype' => 'ol',
			'name'	   => '',
			'orderby'  => 'post_date',
			'order'	   => 'DESC'
		);

		/**
		 * init
		 *
		 * Initialize function: add metabox to the plugin
		 *
		 * @param 	none
		 * @return 	none
		 */
		function init() {

			// Call the parent init function
			parent::init();

			$this->series_data = & EG_Series_Data::get_instance($this->textdomain);

			add_shortcode($this->options['shortcode_list_posts'],  array(&$this,'posts_shortcode') );
			add_shortcode($this->options['shortcode_list_series'], array(&$this,'series_shortcode') );

			if ($this->options['shortcode_auto']>0) {
				add_filter('the_content', array(&$this, 'posts_shortcode_auto'));
			}
		}

		/**
		 * series_shortcode
		 *
		 * Implement a shortcode to get the list of the series
		 *
		 * @param 	array		$attr		list of parameters sent to the shortcde
		 * @return 	string			html code
		 */
		function series_shortcode($attr) {

			extract( shortcode_atts( $this->eg_series_shortcode_series_defaults, $attr ) );
			$string = '';
			$string = $this->series_data->get_series($this->options['custom_field_name'],
												$listtype,			/* list_type		  */
												0, 					/* width 			  */
												$number, 			/* height			  */
												$title,				/* title			  */
												$titletag,			/* tag for title  	  */
												'series_list',		/* Select/option name */
												'',					/* default value      */
												TRUE, 				/* auto_submit	  	  */
												$more,				/* more_series		  */
												FALSE);				/* add no_serie 	  */
			return ($string);
		}  /* End of series_shortcode */

		/**
		 * posts_shortcode
		 *
		 * Implement a shortcode to get the list of posts for a specified serie
		 *
		 * @param 	array		$attr		list of parameters sent to the shortcde
		 * @return 	string					html code
		 */
		function posts_shortcode($attr) {
			global $post;

			extract( shortcode_atts( $this->eg_series_shortcode_posts_defaults, $attr ));
			if ($id == 0) $id = $post->ID;

			return ($this->series_data->get_posts($this->options['custom_field_name'],	/* Meta key */
												$name,									/* serie name */
												$id,									/* or post_id */
												$listtype,
												0,
												0,
												$title,
												$titletag,
												TRUE,
												$order)
											);

		} /* End of posts_shortcode */


		/**
		 * posts_shortcode_auto
		 *
		 * Implement a shortcode to get the list of posts for a specified serie
		 *
		 * @param 	strong	$content	post_content
		 * @return 	string			modified post content
		 */
		function posts_shortcode_auto($content = '') {
			global $post;

			if ( (is_single() || is_page()) && $this->options['shortcode_auto']>0) {
				$content .= $this->series_data->get_posts($this->options['custom_field_name'],		/* meta key 		*/
														'', 										/* Serie name 		*/
														$post->ID, 									/* id of current post */
														$this->options['shortcode_auto_listtype'],	/* type of list		*/
														0,											/* max width 		*/
														0,											/* height			*/
														$this->options['shortcode_auto_title'], 	/* title			*/
														$this->options['shortcode_auto_title_tag'], /* title tag		*/
														FALSE,										/* auto-submit		*/
														'DESC');									/* order			*/
			}
			return ($content);
		}
	} /* End of Class */
} /* End of if class_exists */

/* ---------------------------------------------------------------------------------------------------------
     --- Plugin instance creation
     ------------------------------------------------------------------------------------------------------- */
$eg_series = new EG_Series(EG_SERIES_PLUGIN_NAME, EG_SERIES_VERSION, EG_SERIES_CORE_FILE);
$eg_series->set_textdomain(EG_SERIES_TEXTDOMAIN);
// $eg_series->set_stylesheets('eg-series.css', FALSE);
$eg_series->set_owner(EG_SERIES_AUTHOR_NAME, EG_SERIES_AUTHOR_URL, EG_SERIES_AUTHOR_EMAIL);
$eg_series->set_options(EG_SERIES_OPTIONS_ENTRY, $EG_SERIES_DEFAULT_OPTIONS);
$eg_series->set_wp_versions('2.5',	FALSE, FALSE, FALSE);
$eg_series->active_cache(3600);
$eg_series->load();
?>