<?php
/*
Plugin Name: EG-Metabox
Plugin URI:
Description: Framework for plugin development
Version: 1.00
Author: Emmanuel GEORJON
Author URI: http://www.emmanuelgeorjon.com/
*/

/*
    Copyright 2009 Emmanuel GEORJON  (email : blog@georjon.eu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/* Source of information about metabox:
	- http://wefunction.com/2008/10/tutorial-creating-custom-write-panels-in-wordpress/
	- http://codex.wordpress.org/Function_Reference/add_meta_box
*/

if (!class_exists('EG_MetaBoxes_100')) {

	/**
	  * Class EG_MetaBoxes
	  *
	  * Provide some functions to create a WordPress plugin
	  *
	 */

	Class EG_MetaBoxes_100 {

		var $metaboxes;
		var $textdomain;

		/**
		  * Class contructor
		  *
		  *
		  * @package EG-MetaBoxes
		  * @return object
		  */
		function EG_MetaBoxes_100($textdomain) {

			register_shutdown_function(array(&$this, '__destruct'));
			$this->__construct($textdomain);
		}

		/**
		  * Class contructor
		  *
		  *
		  * @package EG-MetaBoxes
		  * @return object
		  */
		function __construct($textdomain) {

			$this->textdomain = $textdomain;
			$this->metaboxes = array();
		}

		/**
		 * Class destructor
		 *
		 * @package EG-MetaBoxes
		 * @return boolean true
		 */
		function __destruct() {
			// Nothing for the moment
		}

		/**
		  * add_metabox
		  *
		  * Add a metabox to the list
		  *
		  * @package EG-MetaBoxes
		  * @return int	$id	the id of the created metabox
		  */
		function add_metabox($title) {
			$id = 'eg-metaboxes-'.sizeof($this->metaboxes);
			$this->metaboxes[$id]->title = $title;
			$this->metaboxes[$id]->form = array();
			return ($id);
		}

		/**
		  * add_metabox_field
		  *
		  * Add a new field to the specified metabox
		  *
		  * @package EG-MetaBoxes
		  * @param	string	$id				id of the metabox we want to complete
		  * @param	string	$type			type of the field (select, text, radio, checkbox)
		  * @param	string	$name			id of the field (will be saved as-is in the custom fields
		  * @param	string	$default_value	value used when data doesn't exist
		  * @param	string	$description	description of the field (will be displayed in the block
		  * @param  array	$list_values	for radio, checkbox and select. List of the values to display for selection
		  *
		  * @return none
		  */
		function add_metabox_field($id, $type, $name, $title, $default_value, $default_field, $description, $list_values = FALSE) {
			$index = sizeof($this->metaboxes[$id]->form);
			$this->metaboxes[$id]->form[$index]->type          = $type;
			$this->metaboxes[$id]->form[$index]->name          = $name;
			$this->metaboxes[$id]->form[$index]->title         = $title;
			$this->metaboxes[$id]->form[$index]->default_value = $default_value;
			$this->metaboxes[$id]->form[$index]->default_field = $default_field;
			$this->metaboxes[$id]->form[$index]->list_values   = $list_values;
			$this->metaboxes[$id]->form[$index]->description   = $description;
		}

		/**
		  * init
		  *
		  * Link the object methods with WordPress.
		  *
		  * @package EG-MetaBoxes
		  * @param	none
		  * @return none
		  */
		function init() {
		
			if (sizeof($this->metaboxes)>0) {
				add_action('admin_menu', array(&$this, 'create_metaboxes'));
				add_action('save_post',  array(&$this, 'save_metaboxes'), 1, 2);				
			}
		}

		/**
		  * create_metabox
		  *
		  * Create all declared metaboxes
		  *
		  * @package EG-MetaBoxes
		  * @param	none
		  * @return none
		  */
		function create_metaboxes() {

			foreach ($this->metaboxes as $key => $metabox) {
				add_meta_box( $key, __($metabox->title, $this->textdomain), array(&$this, 'display_metabox'), 'post', 'normal', 'high' );
			}
		}

		/**
		 * get_single_field
		 *
		 * Get the different field by requesting the database (get_post_meta)
		 *
		 * @param 	int		$post_id	id of the current post
		 * @param 	object	$field	current field to get value
		 * @return 	string			the value
		 */
		function get_single_field($post_id, $field) {
			// Get the field value
			$value = '';
			if ($field->default_field) $value = get_post_meta($post_id, $field->default_field, true);
			if($value == '') $value = $field->default_value;

			return ($value);
		}

		/**
		  * display_metabox
		  *
		  * Display all declared metaboxes
		  *
		  * @package EG-MetaBoxes
		  * @param	object	$post			the current post
		  * @param	array		$meta_params	some parameters of the metabox, especially the ID
		  * @return none
		  */
		function display_metabox($post, $meta_params = FALSE) {

			// if $meta_params is not specified, use the first, else use the specified
			if (!$meta_params) $current_metabox = current($this->metaboxes);
			else $current_metabox = $this->metaboxes[$meta_params['id']];

			$field_number = 0;
			// for each metabox to display
			foreach($current_metabox->form as $field) {
				// Add a field for the security
				if ($field_number == 0) {
					echo'<input type="hidden" name="'.$field->name.'_noncename" id="'.$field->name.'_noncename" value="'.wp_create_nonce( plugin_basename(__FILE__) ).'" /><table>';
					$field_number++;
				}

				$meta_box_value = $this->get_single_field($post->ID, $field);

				// Display the label
				echo'<tr><td><label for="'.$field->name.'">'.__($field->title, $this->textdomain).'</label></td>';

				// Display the field itself
				switch ($field->type) {
					case 'select':
						$select_string ='<select name="'.$field->name.'">';
						foreach ($field->list_values as $key => $value) {
							if ($key == $meta_box_value) $str = ' selected'; else $str = '';
							$select_string .= '<option value="'.$key.'" '.$str.'>'.$value.'</option>';
						}
						$select_string .='</select>';
						echo '<td>'.$select_string.'</td>';
						break;
					case 'text':
						echo '<td><input type="text" name="'.$field->name.'" value="'.$meta_box_value.'" size="30" /></td>';
						break;
					case 'radio':
						$values = split(',', $field->list_values);
						$select_string ='';
						foreach ($values as $value) {
							if ($value == $meta_box_value) $str = ' checked'; else $str = '';
							$select_string .= '<input type="radio" name="'.$field->name.'" value="'.$value.'" '.$str.' /> '.$value.'<br />';
						}
						$select_string .='</select>';
						echo '<td>'.$select_string.'</td>';
				}

				// Display the description
				echo '</tr>';
				if ($field->description != '') echo '<tr><td>&nbsp;</td><td>'.__($field->description, $this->textdomain).'<br />&nbsp;</td></tr>';
			}
			echo '</table>';
		} /* End of method display_metabox */

		/**
		  * save_single_field
		  *
		  * Save a field value to a 'custom field'
		  *
		  * @package EG-MetaBoxes
		  * @param	int		$post_id			id of the current post
		  * @param	string	$name			name of the field to save
		  * @param	string	$data			current data to save (get from the form)
		  * @param 	string	$previous_data		data previously saved in custom field
		  * @return none
		  */
		function save_single_field($post_id, $name, $data, $previous_data) {

			if ($data == '') {
				delete_post_meta($post_id, $name, $previous_data);
			} else {
				if ($previous_data == '') add_post_meta($post_id, $name, $data, true);
				elseif ($data != $previous_data) update_post_meta($post_id, $name, $data);
			}
		} /* End of method save_single_field */

		/**
		  * save_all_fields
		  *
		  * Save all fields values to 'custom fields'
		  *
		  * @package EG-MetaBoxes
		  * @param	int		$post_id		id of the current post
		  * @param 	array		$meta_box		a metabox
		  * @return none
		  */
		function save_all_fields($post_id, $meta_box) {

			//  For each metaboxes, get the list of the field
			foreach ($meta_box->form as $field) {

				// Get the data from the form, and the previously saved data
				$data          = $_POST[$field->name];
				$previous_data = get_post_meta($post_id, $field->name);
				$this->save_single_field($post_id, $field->name, $data, $previous_data);
			}
		} /* End of method save_all_fields */

		/**
		  * save_metabox
		  *
		  * Save all fields values for all metaboxes displayed
		  *
		  * @package EG-MetaBoxes
		  * @param	int		$post_id		id of the current post
		  * @return none
		  */
		function save_metaboxes($post_id, $post) {
	
			// Since 2.6, we have to manage revisions. save_post hook is called twice: one for the revision, and one for the original post
			// Nothing to do on the revision !!!
			// See post in http://alexking.org/blog/2008/09/06/wordpress-26x-duplicate-custom-field-issue
			if ($post->post_type == 'revision')
				return;

			// check if we have the enough privilege to save the metaboxes values
			if ( 'page' == $_POST['post_type'] ) {
				if ( !current_user_can( 'edit_page', $post_id ))
					return $post_id;
			} else {
				if ( !current_user_can( 'edit_post', $post_id ))
					return $post_id;
			}
			// For each metabox ...
			foreach($this->metaboxes as $key => $meta_box) {

				// Check security by collecting noncename
				$first_field = current($meta_box->form );
				if ( !wp_verify_nonce( $_POST[$first_field->name.'_noncename'], plugin_basename(__FILE__) )) {
					return $post_id;
				}
				else {
					// It's Ok, we can save.
					$this->save_all_fields($post_id, $meta_box);
				}
			}
		} /* End of method save_metabox */
	} /* End of class */
} /* End of class_exists */
